#include <string>
#include <list>
#include "komb.h"
using namespace std;

const int MAX_SIZE=30;                          // maximal number of vertices

typedef int AdjMatrix[MAX_SIZE+1][MAX_SIZE+1];	// typedef for adjacency matrix



//Path - representing path as an array of vertices
class Path
{
   int P[MAX_SIZE+1];                     // array of consecutive vertices of a path
   int size;                              // number of vertices in a path
  public:   
   Path(const int A[], int k);            // constructor: array of vertices and its size
   void showPath();					      // display a path
   bool hasCommonVertex(int C[], int setSize);               // check if path has a common vertex with C[1], C[2], ... C[setSize]
   bool hasCommonVertex2(int *C, int setSize, int nVerts);   //VERSION 2 - additional parameter: number of vertices in the considered graph

//VERSION 3
  private:
   hash_t hash;                           // value of "hash function" for a path (bit array of vertices)
  public:
   int firstVertex();                     // first vertex of a path
   int lastVertex();                      // last vertex of a path
   void addFirstVertex(int v);            // add new vertex at the beginning
   void addLastVertex(int v);             // add new vertex at the end
   bool contains(int v);                  // check if path contains a vertex
   hash_t getHash();                      // return hash function value
   bool hasCommonVertex3(hash_t set, int setSize);   //VERSION 3 - compare hash values
};


//Graph - reperesenting graph as an adjacency matrix
class Graph
{
  public:
    AdjMatrix A;                          // adjacency matrix
    int graphSize;                        // number of vertices of a graph
    int pathSize;                         // number of vertices in currently considered paths
    int PathSequence[MAX_SIZE+1];         // path sequence of a graph
    list <Path> *PathList;                // list of paths of currently considered size
    
    Graph (string s, int size);           // constructor: creates a graph from the string containing its adjacency matrix
                                          //  in Mathematica text format; size  the number of vertices
    ~Graph();                             // destructor

    void showMatrix();                    // for displaying the adjacency matrix in more redable form

//VERSION 1
    bool isPath(int B[]);                 // check whether the array of vertices B forms a path in a graph
    void genPaths(int k);                 // generete all paths of size k in a graph and store them in PathList
    bool isPathVertexCover(int setSize);  // check if C is a k-path vertex cover of a graph
    void calculatePsi (int k);            // calculate the value of psi_k
    void calculatePathSequence();         // calculate path sequence (psi_1, psi_2, ... )
    void showPathSequence();              // show calculated path sequence

    int C[MAX_SIZE+1];				      // currently considered candidate for path vertex cover; used for generation of permutations and other functions
    void swap(int i,int j);               // help functions for generating permutations
    void reverse (int m);
    void perm (int m);

//VERSION 2
    bool isPath2(int* P, int k);          // check whether the sequence of vertices P[0], P[1], ... P[k-1] forms a path
    void genPaths2(int k, int** set);     // generate all paths on k vertices in a graph, using array of k-permutations and store them in PathList
    bool isPathVertexCover2(int *S, int setSize);   // check if S (a set of setSize vertices) is a path vertex cover of a graph
    int calculatePsi2(int k, int*** setArray);      // calculate psi_k checking all the sets in setArray[k] as possible path vertex covers
    void calculatePathSequence2(int*** setArray, int*** permArray);  // calculate path sequence using arrays of all sets and permutations

//VERSION 3
    list<list<Path> > *PathList3;         // list of list of paths - all paths on one list share the same hash function value (i.e. have the same set of vertices)

    void genPaths3(int k);                // generate all paths on k vertices in a graph and store them in PathList3; if k>2 we assume that PathList3 contains paths of length (k-1) before using this function
    bool isPathVertexCover3(hash_t set, int setSize);  // check if the set with hash function value of set is a k-path vertex cover of a graph
    int calculatePsi3(int k);             // calculate psi_k checking all the sets in setArray[k] as possible path vertex covers
    void calculatePathSequence3();        // calculate path sequence using arrays of all sets and genPaths3
};

